package com.javacodegeeks.elasticsearch.hibernate;

import java.util.List;

import javax.persistence.EntityManager;
import javax.transaction.Transactional;

import org.hibernate.search.jpa.FullTextEntityManager;
import org.hibernate.search.jpa.FullTextQuery;
import org.hibernate.search.jpa.Search;
import org.hibernate.search.query.dsl.QueryBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.javacodegeeks.elasticsearch.hibernate.model.Author;
import com.javacodegeeks.elasticsearch.hibernate.model.Book;

@Service
public class CatalogService {
	@Autowired private EntityManager entityManager;
	
	@Transactional
	public void addBook(Book book) {
		entityManager.persist(book);
	}

	@Transactional
	public void addAuthor(Author author) {
		entityManager.persist(author);
	}

	@SuppressWarnings("unchecked")
	@Transactional
	public List<Book> searchByCategoryAndAuthor(String name, String author) {
		final FullTextEntityManager fullTextEntityManager = Search
			.getFullTextEntityManager(entityManager);
		
		final QueryBuilder qb = fullTextEntityManager
			.getSearchFactory()
			.buildQueryBuilder()
			.forEntity(Book.class)
			.get();
		
		final FullTextQuery query = fullTextEntityManager
			.createFullTextQuery(
				qb
					.bool()
					.must(
						qb.keyword()
		                	.onField("categories.name")
		                	.matching(name)
		                	.createQuery()
		            )
					.must(
						qb.keyword()
		                	.onField("authors.last_name")
		                	.matching(author)
		                	.createQuery()
		            )
                	.createQuery(),
                Book.class 
			);
		
		return query.getResultList();
	}
}
