package com.javacodegeeks.elasticsearch.client;

import static org.assertj.core.api.Assertions.assertThat;

import java.time.LocalDate;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.javacodegeeks.elasticsearch.hibernate.CatalogService;
import com.javacodegeeks.elasticsearch.hibernate.ElasticsearchHibernateConfiguration;
import com.javacodegeeks.elasticsearch.hibernate.model.Author;
import com.javacodegeeks.elasticsearch.hibernate.model.Book;
import com.javacodegeeks.elasticsearch.hibernate.model.Category;

@SpringBootTest(classes = ElasticsearchHibernateConfiguration.class)
@RunWith(SpringJUnit4ClassRunner.class)
public class ElasticsearchHibernateTest {
	@Autowired private CatalogService catalogService;

	@Test
	public void esHibernateIndexBook() {
		final Author clintonGormley = new Author("1");
		clintonGormley.setFirstName("Clinton");
		clintonGormley.setLastName("Gormley");
		catalogService.addAuthor(clintonGormley);
		
		final Author zacharyTong = new Author("2");
		zacharyTong.setFirstName("Zachary");
		zacharyTong.setLastName("Tong");
		catalogService.addAuthor(zacharyTong);
		
		final Book book = new Book("978-1449358549");
		book.setTitle("Elasticsearch: The Definitive Guide. A Distributed Real-Time Search and Analytics Engine");
		book.addCategory(new Category("analytics"));
		book.addCategory(new Category("search"));
		book.addCategory(new Category("database store"));
		book.setPublisher("O'Reilly");
		book.setDescription("Whether you need full-text search or real-time analytics of structured dataвЂ”or bothвЂ”the Elasticsearch distributed search engine is an ideal way to put your data to work. This practical guide not only shows you how to search, analyze, and explore data with Elasticsearch, but also helps you deal with the complexities of human language, geolocation, and relationships.");
		book.setPublishedDate(LocalDate.of(2015, 02, 07));
		book.setRating(4);
		
		book.addAuthor(clintonGormley);
		book.addAuthor(zacharyTong);
		
		catalogService.addBook(book);
		assertThat(catalogService.searchByCategoryAndAuthor("analytics", "Tong")).isNotEmpty();
	}
}
